/* Copyright 2010-2011, Unpublished Work of Technologic Systems
 * All Rights Reserved.
 *
 * THIS WORK IS AN UNPUBLISHED WORK AND CONTAINS CONFIDENTIAL,
 * PROPRIETARY AND TRADE SECRET INFORMATION OF TECHNOLOGIC SYSTEMS.
 * ACCESS TO THIS WORK IS RESTRICTED TO (I) TECHNOLOGIC SYSTEMS 
 * EMPLOYEES WHO HAVE A NEED TO KNOW TO PERFORM TASKS WITHIN THE SCOPE
 * OF THEIR ASSIGNMENTS AND (II) ENTITIES OTHER THAN TECHNOLOGIC
 * SYSTEMS WHO HAVE ENTERED INTO APPROPRIATE LICENSE AGREEMENTS.  NO
 * PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED, COPIED, 
 * DISTRIBUTED, REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED, 
 * EXPANDED, COLLECTED, COMPILED, LINKED, RECAST, TRANSFORMED, ADAPTED
 * IN ANY FORM OR BY ANY MEANS, MANUAL, MECHANICAL, CHEMICAL, 
 * ELECTRICAL, ELECTRONIC, OPTICAL, BIOLOGICAL, OR OTHERWISE WITHOUT
 * THE PRIOR WRITTEN PERMISSION AND CONSENT OF TECHNOLOGIC SYSTEMS.
 * ANY USE OR EXPLOITATION OF THIS WORK WITHOUT THE PRIOR WRITTEN
 * CONSENT OF TECHNOLOGIC SYSTEMS COULD SUBJECT THE PERPETRATOR TO
 * CRIMINAL AND CIVIL LIABILITY.
 */

module ts4800_top(
  eim_d_pad,
  eim_bclk_pad,
  eim_lba_pad,
  eim_rd_padn,
  eim_wr_padn,
  eim_wait_padn,
  eim_cs0_padn,
  eim_cs1_padn,
  eim_byte0_padn,
  eim_byte1_padn,
  eim_dtack_pad,
  eim_gpio2_9_pad,
  ext_reset_padn,
  fpga_24mhz_pad,
  low_volt_padn,

  mux_ad_pad,
  bus_ale_padn,
  bus_dir_pad,
  bus_cs_padn,
  bus_bhe_padn,
  bus_wait_padn,
  dio_pad,
  uart5_rxd_pad,
  off_bd_rst_padn,
      
  por_padn,
  reset_in_padn,
  cpu_32khz_pad,
  strap_low_pad,
  pmic_on_od_pad,
  ext_temp_padn,
  usb_24mhz_pad,
  pmic_stby_req18v_pad,
  can_rxd_pad, 
  can_txd_pad, 
  uart5_txd_pad,
  sd_power_padn,
  spi_mosi_pad,
  spi_cs_padn,
  spi_clk_pad,
  spi_miso_pad,
  uart0_txd_pad,
  uart0_rxd_pad,
  uart3_rxd_pad,
  uart4_txd_pad,
  uart4_rxd_pad,
  uart3_txd_pad,
  reset_fpga_padn,

  red_led_padn,
  green_led_padn,
  nc_pad
);

inout [15:0] eim_d_pad;
input eim_bclk_pad;
input eim_lba_pad;
input eim_rd_padn;
input eim_wr_padn;
output eim_wait_padn;
input eim_cs0_padn;
input eim_cs1_padn;
input eim_byte0_padn;
input eim_byte1_padn;
inout eim_dtack_pad;	//CPU Interrupt
inout eim_gpio2_9_pad;	//CPU Interrupt
input ext_reset_padn;  
input fpga_24mhz_pad;
input low_volt_padn;

inout [15:0] mux_ad_pad;
inout bus_ale_padn;
inout bus_dir_pad;
inout bus_cs_padn;
inout bus_bhe_padn;
inout bus_wait_padn;
inout uart5_rxd_pad;
inout [20:0] dio_pad;
output off_bd_rst_padn;
      
output por_padn;
output reset_in_padn;
input cpu_32khz_pad;
output strap_low_pad;
output pmic_on_od_pad;
input ext_temp_padn;
output usb_24mhz_pad;
output pmic_stby_req18v_pad;
inout can_rxd_pad;
inout can_txd_pad;
inout uart5_txd_pad;
output sd_power_padn;
inout spi_mosi_pad;
inout spi_cs_padn;
inout spi_clk_pad;
inout spi_miso_pad;

inout uart0_txd_pad;
inout uart0_rxd_pad;
inout uart3_rxd_pad;
inout uart4_txd_pad;
inout uart4_rxd_pad;
inout uart3_txd_pad;
output reset_fpga_padn;

output red_led_padn;
output green_led_padn;

input [2:0] nc_pad;

parameter externalbus_opt = 1'b1;
parameter touchscreen_opt = 1'b1;
parameter spi_opt = 1'b1;
parameter xuart_opt = 1'b1;
parameter can0_opt = 1'b1;
parameter can1_opt = 1'b1;
/*
parameter externalbus_opt = 1'b0;
parameter touchscreen_opt = 1'b0;
parameter spi_opt = 1'b0;
parameter xuart_opt = 1'b0;
parameter can0_opt = 1'b0;
parameter can1_opt = 1'b0;
*/

/****************************************************************************
 * Boilerplate FPGA configuration (clocks, PLL, DLL, reset) 
 ****************************************************************************/

wire clk_130mhz, clk_100mhz, clk_300mhz, clk_75mhz, clk_13mhz;
wire wb_clk = clk_75mhz;

wire [4:0] pllphase;
wire lock, lock2;
pll clkgen (
  .CLK(fpga_24mhz_pad),
  .CLKOP(clk_300mhz),
  .CLKOK(clk_75mhz),
  .CLKOK2(clk_100mhz),
  .LOCK(lock)
);
pll2 clkgen2 (
  .CLK(clk_300mhz),
  .CLKOP(clk_130mhz), // not used
  .CLKOK(clk_13mhz),
  .LOCK(lock2)
);

assign usb_24mhz_pad = clk_13mhz; // USB3318 PHY requires 13MHz ref clk
// If you have TS-4800 rev A PCB, you need to do this instead:
// assign usb_24mhz_pad = fpga_24mhz_pad;


/****************************************************************************
 * RESET SIGNALS
 ****************************************************************************/
wire reset = !low_volt_padn;
reg boot_mode;
always @(*) begin
  boot_mode = reset ? bus_dir_pad : boot_mode;
end

wire xuart_irq, can0_irqn, can1_irqn;
wire irq_controller_o, irq_can1;
assign eim_dtack_pad = (!can0_irqn && can0_opt);
assign eim_gpio2_9_pad = irq_can1 ? (!can1_irqn && can1_opt) : irq_controller_o;

reg [15:0] rst_counter = 16'd0;
wire rstn = rst_counter[15];
reg reset_pll_q;
always @(posedge wb_clk) begin
  reset_pll_q <= lock && lock2;
  if (!rst_counter[15]) begin
    if (reset_pll_q) rst_counter <= rst_counter + 1'b1;
    else rst_counter <= 16'd0;
  end
end
wire wb_rst = !rstn;
reg [31:0] timer;
reg unreset;
reg mode1, mode2, mode3;
always @(posedge wb_clk or posedge wb_rst) begin
  if (wb_rst) begin
    timer <= 32'd0;
    unreset <= 1'b0;
    mode1 <= 1'b1;
    mode2 <= 1'b1;
    mode3 <= 1'b1;
  end else begin
    timer <= timer + 1'b1;
    if (timer >= 32'd64) begin
      unreset <= 1'b1;
      if (!unreset) begin
        ///?mode1 <= dio_pad[26];
        ///?mode2 <= dio_pad[25];
        ///?mode3 <= gpio_a22_pad;
      end
    end
  end
end

reg clk_12mhz;
always @(posedge fpga_24mhz_pad or posedge wb_rst) begin
  if (wb_rst) clk_12mhz <= 1'b0;
  else clk_12mhz <= !clk_12mhz;
end

/****************************************************************************
 * WEIM -> WISHBONE bridge
 ****************************************************************************/
wire weimwbm_cyc_o, weimwbm_stb_o, weimwbm_we_o;
wire [31:0] weimwbm_adr_o;
wire [15:0] weimwbm_dat_o;
wire [3:0] weimwbm_sel_o;
reg [15:0] weimwbm_dat_i;
reg weimwbm_ack_i;
wire [15:0] eim_d_o;
wire eim_d_oe_o, eim_wait_n_o;
wire [16:0] ts4800_decodes;
weim_wishbone_bridge weimwbm(
  .wbm_clk_i(wb_clk),
  .wbm_adr_o(weimwbm_adr_o),
  .wbm_cyc_o(weimwbm_cyc_o),
  .wbm_stb_o(weimwbm_stb_o),
  .wbm_we_o(weimwbm_we_o),
  .wbm_dat_o(weimwbm_dat_o),
  .wbm_sel_o(weimwbm_sel_o),
  .wbm_dat_i(weimwbm_dat_i),
  .wbm_ack_i(weimwbm_ack_i),

  .rst_i(wb_rst),
  .eim_d_i(eim_d_pad),
  .eim_d_o(eim_d_o),
  .eim_d_oe_o(eim_d_oe_o),
  .eim_bclk_i(eim_bclk_pad),
  .eim_lba_i(eim_lba_pad),
  .eim_rd_n_i(eim_rd_padn),
  .eim_wr_n_i(eim_wr_padn),
  .eim_wait_n_o(eim_wait_n_o),
  .eim_cs0_n_i(eim_cs0_padn),
  .eim_cs1_n_i(eim_cs1_padn),
  .eim_byte0_n_i(eim_byte0_padn),
  .eim_byte1_n_i(eim_byte1_padn),
  
  .ts4800_decodes_o(ts4800_decodes)
);
assign eim_d_pad = eim_d_oe_o ? eim_d_o : 16'hzzzz;
assign eim_wait_padn = eim_wait_n_o;


/****************************************************************************
 * TS-XUART super uarts
 ****************************************************************************/
reg xuwbs_en;
wire [15:0] xuartwbs_dat_o;
wire xuartwbs_ack_o;
wire uartwbm_cyc_o, uartwbm_stb_o, uartwbm_we_o, uartwbm_ack_i;
wire [31:0] uartwbm_adr_o;
wire [15:0] uartwbm_dat_o;
wire [15:0] uartwbm_dat_i;
wire [3:0] uartwbm_sel_o;
reg [7:0] xuart_rx;
wire [7:0] xuart_active;
wire [7:0] xuart_cts = 8'hff;
wire [7:0] xuart_txen, xuart_tx;
reg [7:0] xuart_rx_q, xuart_cts_q;
ts_xuart tsxuartcore(
  .wb_clk_i(wb_clk & xuart_opt),
  .wb_rst_i(wb_rst),
  .wbm_cyc_o(uartwbm_cyc_o),
  .wbm_stb_o(uartwbm_stb_o),
  .wbm_we_o(uartwbm_we_o),
  .wbm_dat_o(uartwbm_dat_o),
  .wbm_adr_o(uartwbm_adr_o),
  .wbm_ack_i(uartwbm_ack_i),
  .wbm_dat_i(uartwbm_dat_i),

  .wbs_cyc_i(weimwbm_cyc_o && xuwbs_en),
  .wbs_stb_i(weimwbm_stb_o && xuwbs_en),
  .wbs_we_i(weimwbm_we_o),
  .wbs_adr_i(weimwbm_adr_o),
  .wbs_dat_i(weimwbm_dat_o),
  .wbs_dat_o(xuartwbs_dat_o),
  .wbs_ack_o(xuartwbs_ack_o),

  .baudclk_i(clk_100mhz),
  .cts_i(xuart_cts_q),
  .rx_i(xuart_rx_q),
  .tx_o(xuart_tx),
  .txen_o(xuart_txen),
  .active_o(xuart_active),
  .irq_o(xuart_irq),
  .uart_disable_i(8'hf0)
);
assign uartwbm_sel_o = uartwbm_adr_o[1] ? 4'b1100 : 4'b0011;
always @(posedge wb_clk or posedge wb_rst) begin
  if (wb_rst) begin
    xuart_rx_q <= 8'hff;
    xuart_cts_q <= 8'hff;
  end else begin
    xuart_rx_q <= xuart_rx;
    xuart_cts_q <= xuart_cts;
  end
end


/****************************************************************************
 * TS-BOOTROM blockram with WEIM-WISHBONE interface 
 ****************************************************************************/
reg ramwbs1_en;
wire ramwbs1_ack_o;
wire [31:0] ramwbs1_dat_o;
wire [15:0] ramwbs1_dat16_o;
reg ramwbs2_en;
wire ramwbs2_ack_o;
wire [31:0] ramwbs2_dat_o;
wire [15:0] ramwbs2_dat16_o;
reg ramwbs3_en;
wire ramwbs3_ack_o;
wire [31:0] ramwbs3_dat_o;
wire [3:0] ramwbs1_sel_i = weimwbm_adr_o[1] ? 
  {weimwbm_sel_o, 2'b00} : {2'b00, weimwbm_sel_o};
blockram_8kbyte sramcore(
  .wb_clk_i(wb_clk),
  .wb_rst_i(wb_rst),

  .wb1_cyc_i(weimwbm_cyc_o && ramwbs1_en),
  .wb1_stb_i(weimwbm_stb_o && ramwbs1_en),
  .wb1_we_i(weimwbm_we_o),
  .wb1_dat_i({weimwbm_dat_o, weimwbm_dat_o}),
  .wb1_adr_i({weimwbm_adr_o[31:2], 2'b00}),
  .wb1_sel_i(ramwbs1_sel_i),
  .wb1_dat_o(ramwbs1_dat_o),
  .wb1_ack_o(ramwbs1_ack_o),

  .wb2_cyc_i(uartwbm_cyc_o),
  .wb2_stb_i(uartwbm_stb_o),
  .wb2_we_i(uartwbm_we_o),
  .wb2_dat_i({uartwbm_dat_o, uartwbm_dat_o}),
  .wb2_adr_i({16'h0000, uartwbm_adr_o[15:2], 2'b00}),
  .wb2_sel_i(uartwbm_sel_o),
  .wb2_dat_o(ramwbs2_dat_o),
  .wb2_ack_o(ramwbs2_ack_o)
);

assign ramwbs1_dat16_o[15:0] = weimwbm_adr_o[1] ? ramwbs1_dat_o[31:16] : ramwbs1_dat_o[15:0];
assign ramwbs2_dat16_o[15:0] = uartwbm_adr_o[1] ? ramwbs2_dat_o[31:16] : ramwbs2_dat_o[15:0];

assign uartwbm_ack_i = ramwbs2_ack_o;
assign uartwbm_dat_i = ramwbs2_dat16_o; 


/****************************************************************************
 * SYSCON (misc control registers)
 ****************************************************************************/
reg scwbs_en;
wire [15:0] scwbs_dat_o;
wire scwbs_ack_o;
wire [55:0] dio_o;
wire [55:0] dio_oe_o;
wire [55:0] dio_i;
wire reset_fpga, sd_power, bbclk, led_red, led_grn;
wire [15:0] bus_config;
wire [15:0] core_enable;
wire can0_wbaccess, can1_wbaccess;
wire en_ts8510_rs422, en_rs422;
wire [1:0] counter_en;
wire [31:0] usec;
syscon sysconcore(
  .wb_clk_i(wb_clk),
  .wb_rst_i(wb_rst),
  .wb_cyc_i(weimwbm_cyc_o && scwbs_en),
  .wb_stb_i(weimwbm_stb_o && scwbs_en),
  .wb_we_i(weimwbm_we_o),
  .wb_adr_i(weimwbm_adr_o),
  .wb_dat_i(weimwbm_dat_o),
  .wb_sel_i(weimwbm_sel_o),
  .wb_dat_o(scwbs_dat_o),
  .wb_ack_o(scwbs_ack_o),

  .dio_i(dio_i),
  .dio_oe_o(dio_oe_o),
  .dio_o(dio_o),

  .mode1_i(mode1),
  .mode2_i(mode2),
  .mode3_i(mode3),
  .pllphase_o(pllphase),
  .timer_i(timer),
  .led_grn_o(led_grn),
  .led_red_o(led_red),
  .boot_mode_i(boot_mode),
  .reset_fpga_o(reset_fpga),
  .sd_power_o(sd_power),
  .bbclk_o(bbclk),
  .irq_can1_o(irq_can1),
  .strap_low_o(strap_low),
  .bus_config_o(bus_config),
  .core_enable_o(core_enable),
  .ext_temp_i(!ext_temp_padn),
  .en_ts8510_rs422_o(en_ts8510_rs422),
  .en_rs422_o(en_rs422),
  .counter_en_o(counter_en),
  .usec_o(usec)
);

assign por_padn = 1'bz;
assign off_bd_rst_padn = reset ? 1'b0 : 1'b1; //= low_volt_padn;
assign reset_in_padn = 1'bz;
assign pmic_on_od_pad = 1'bz;
assign pmic_stby_req18v_pad = 1'bz;

assign red_led_padn = led_red ? 1'b0 : 1'bz;
assign green_led_padn = led_grn ? 1'b0 : 1'bz;
assign reset_fpga_padn = (reset_fpga || !ext_reset_padn) ? 1'b0 : 1'b1;
assign sd_power_padn = !sd_power;
assign strap_low_pad = strap_low;


/****************************************************************************
 * SPI controller logic
 ****************************************************************************/
reg spiwbs_en;
wire [15:0] spiwbs_dat_o;
wire spiwbs_ack_o;
wire scken;
wire sck, hispeed, so;
wire [3:0] csn, si;
reg [3:0] csn_q;

wb_spi spicore(
  .wb_clk_i(wb_clk & spi_opt),
  .wb_rst_i(wb_rst),
  .wb_cyc_i(weimwbm_cyc_o && spiwbs_en),
  .wb_stb_i(weimwbm_stb_o && spiwbs_en),
  .wb_dat_i(weimwbm_dat_o),
  .wb_adr_i(weimwbm_adr_o),
  .wb_we_i(weimwbm_we_o),
  .wb_sel_i(weimwbm_sel_o),
  .wb_dat_o(spiwbs_dat_o),
  .wb_ack_o(spiwbs_ack_o),

  .scken_o(scken),
  .so_o(so),
  .si_i(si),
  .csn_o(csn),
  .sck_o(sck),
  .hispeed_o(hispeed)
);

always @(posedge wb_clk or posedge wb_rst) begin
  if (wb_rst) csn_q <= 4'hf;
  else csn_q <= csn;
end
assign hispeed = 1'b0;
assign si = {4{spi_miso_pad}};


/****************************************************************************
 * CAN0 - SJA1000C compatible CAN controller
 ****************************************************************************/
wire can0_tx;
wire can0wbs_cyc_i, can0wbs_stb_i, can0wbs_we_i, can0wbs_ack, can0wbs_ack_o;
wire [7:0] can0wbs_dat_i, can0wbs_dat_o;
wire [15:0] can0wbs_adr_i;
can_top can0core(
  .wb_clk_i(wb_clk && can0_opt),
  .wb_rst_i(wb_rst),
  .wb_cyc_i(can0wbs_cyc_i),
  .wb_stb_i(can0wbs_stb_i),
  .wb_we_i(can0wbs_we_i),
  .wb_dat_i(can0wbs_dat_i),
  .wb_adr_i(can0wbs_adr_i[7:0]),
  .wb_dat_o(can0wbs_dat_o),
  .wb_ack_o(can0wbs_ack),

  .clk_i(fpga_24mhz_pad && can0_opt),
  .rx_i(can_rxd_pad),
  .tx_o(can0_tx),
  .irq_on(can0_irqn)
);
assign can0wbs_ack_o = can0wbs_ack | !can0_opt;
assign can0_wbaccess = can0wbs_cyc_i && can0wbs_stb_i;

/****************************************************************************
 * CAN1 - SJA1000C compatible CAN controller
 ****************************************************************************/
wire can1_tx;
wire can1wbs_cyc_i, can1wbs_stb_i, can1wbs_we_i, can1wbs_ack, can1wbs_ack_o;
wire [7:0] can1wbs_dat_i, can1wbs_dat_o;
wire [15:0] can1wbs_adr_i;
can_top can1core(
  .wb_clk_i(wb_clk && can1_opt),
  .wb_rst_i(wb_rst),
  .wb_cyc_i(can1wbs_cyc_i),
  .wb_stb_i(can1wbs_stb_i),
  .wb_we_i(can1wbs_we_i),
  .wb_dat_i(can1wbs_dat_i),
  .wb_adr_i(can1wbs_adr_i[7:0]),
  .wb_dat_o(can1wbs_dat_o),
  .wb_ack_o(can1wbs_ack),

  .clk_i(fpga_24mhz_pad && can1_opt),
  .rx_i(dio_pad[11]),
  .tx_o(can1_tx),
  .irq_on(can1_irqn)
);
assign can1wbs_ack_o = can1wbs_ack | !can1_opt;
assign can1_wbaccess = can1wbs_cyc_i && can1wbs_stb_i;


/****************************************************************************
 * CAN0 - WEIM 16to8 bit window (for 256 by 8-bit SJA1000C CAN address space)
 ****************************************************************************/
reg mwcan0wbs_en;
wire [15:0] mwcan0wbs_dat_o;
wire mwcan0wbs_ack_o;
wb_memwindow16to8 mwcan0core(
  .wb_clk_i(wb_clk && can0_opt),
  .wb_rst_i(wb_rst),

  .wb_cyc_i(weimwbm_cyc_o && mwcan0wbs_en),
  .wb_stb_i(weimwbm_stb_o && mwcan0wbs_en),
  .wb_we_i(weimwbm_we_o),
  .wb_adr_i(weimwbm_adr_o),
  .wb_dat_i(weimwbm_dat_o),
  .wb_dat_o(mwcan0wbs_dat_o),
  .wb_ack_o(mwcan0wbs_ack_o),
  
  .wbm_cyc_o(can0wbs_cyc_i),
  .wbm_stb_o(can0wbs_stb_i),
  .wbm_we_o(can0wbs_we_i),
  .wbm_adr_o(can0wbs_adr_i),
  .wbm_dat_o(can0wbs_dat_i),
  .wbm_dat_i(can0wbs_dat_o),
  .wbm_ack_i(can0wbs_ack_o)
);

/****************************************************************************
 * CAN1 - WEIM 16to8 bit window (for 256 by 8-bit SJA1000C CAN address space)
 ****************************************************************************/
reg mwcan1wbs_en;
wire [15:0] mwcan1wbs_dat_o;
wire mwcan1wbs_ack_o;
wb_memwindow16to8 mwcan1core(
  .wb_clk_i(wb_clk && can1_opt),
  .wb_rst_i(wb_rst),

  .wb_cyc_i(weimwbm_cyc_o && mwcan1wbs_en),
  .wb_stb_i(weimwbm_stb_o && mwcan1wbs_en),
  .wb_we_i(weimwbm_we_o),
  .wb_adr_i(weimwbm_adr_o),
  .wb_dat_i(weimwbm_dat_o),
  .wb_dat_o(mwcan1wbs_dat_o),
  .wb_ack_o(mwcan1wbs_ack_o),
  
  .wbm_cyc_o(can1wbs_cyc_i),
  .wbm_stb_o(can1wbs_stb_i),
  .wbm_we_o(can1wbs_we_i),
  .wbm_adr_o(can1wbs_adr_i),
  .wbm_dat_o(can1wbs_dat_i),
  .wbm_dat_i(can1wbs_dat_o),
  .wbm_ack_i(can1wbs_ack_o)
);


/****************************************************************************
 * Touchscreen controller 
 ****************************************************************************/
reg tswbs_en;
wire [15:0] tswbs_dat_o;
wire tswbs_ack_o;
wire touch_miso, touch_mosi, touch_clk;
wire ad7843_csn, ad7843_penirqn;
touchscreen ts(
  .wb_clk_i(wb_clk & touchscreen_opt),
  .wb_rst_i(wb_rst),
  .wb_cyc_i(weimwbm_cyc_o && tswbs_en),
  .wb_stb_i(weimwbm_stb_o && tswbs_en),
  .wb_we_i(weimwbm_we_o),
  .wb_adr_i(weimwbm_adr_o),
  .wb_dat_i(weimwbm_dat_o),
  .wb_dat_o(tswbs_dat_o),
  .wb_ack_o(tswbs_ack_o),

  .spi_miso_i(touch_miso),
  .spi_mosi_o(touch_mosi),
  .spi_clk_o(touch_clk),
  .ad7843_csn_o(ad7843_csn),
  .ad7843_penirqn_i(ad7843_penirqn)
);
assign touch_miso = mux_ad_pad[13];
assign ad7843_penirqn = mux_ad_pad[11];


/****************************************************************************
 * Motor controller 
 ****************************************************************************/
reg motwbs_en;
wire [15:0] motwbs_dat_o;
wire motwbs_ack_o = 1'b1;
// Motor core is not included in opencore.
// If you would like to use the motor core in an opencore design, please
// contact Technologic Systems.

/****************************************************************************
 * ADC core
 ****************************************************************************/
reg adcwbs_en;
wire [15:0] adcwbs_dat_o;
wire adcwbs_ack_o;
wire adc_scl, adc_sda, an_sel, adc_active;
wire [1:0] an_sel_sel;

wb_mcp3428 adc(
  .wb_rst_i(wb_rst),
  .wb_clk_i(wb_clk),
  .wb_cyc_i(weimwbm_cyc_o && adcwbs_en),
  .wb_stb_i(weimwbm_stb_o && adcwbs_en),
  .wb_dat_o(adcwbs_dat_o),
  .wb_dat_i(weimwbm_dat_o),
  .wb_adr_i(weimwbm_adr_o),
  .wb_we_i(weimwbm_we_o),
  .wb_ack_o(adcwbs_ack_o),
  .wb_sel_i(weimwbm_sel_o),

  .scl_o(adc_scl),
  .sda_o(adc_sda),
  .sda_i(dio_pad[6]),
  .an_sel_o(an_sel),
  .an_sel_sel_o(an_sel_sel),
  .active_o(adc_active)
);

/****************************************************************************
 * General purpose memory window for external bus
 ****************************************************************************/
wire mwinwbm_cyc_o, mwinwbm_stb_o, mwinwbm_we_o;
reg mwinwbm_ack_i; 
wire [31:0] mwinwbm_adr_o;
wire [31:0] mwinwbm_dat_o;
reg [31:0] mwinwbm_dat_i;
wire [3:0] mwinwbm_sel_o;
reg mwinwbs_en;
wire [15:0] mwinwbs_dat_o;
wire mwinwbs_ack_o;
reg [1:0] mwinbuswidth;

wire [31:0] buf_weimwbm_dat_o, buf_mwinwbs_dat_o, buf_weimwbm_adr_o;
wire [3:0] buf_weimwbm_sel_o;
wire buf_mwinwbs_ack_o, buf_weimwbm_cyc_o, buf_weimwbm_stb_o, buf_weimwbm_we_o;

wb_buf buf1(
  .wb_clk_i(wb_clk),
  .wb_rst_i(wb_rst),

  .wbs_adr_i(weimwbm_adr_o),
  .wbs_cyc_i(weimwbm_cyc_o && mwinwbs_en),
  .wbs_stb_i(weimwbm_stb_o && mwinwbs_en),
  .wbs_we_i(weimwbm_we_o),
  .wbs_dat_i(weimwbm_dat_o),
  .wbs_dat_o(mwinwbs_dat_o),
  .wbs_ack_o(mwinwbs_ack_o),
  .wbs_sel_i(weimwbm_sel_o),

  .wbm_cyc_o(buf_weimwbm_cyc_o),
  .wbm_stb_o(buf_weimwbm_stb_o),
  .wbm_we_o(buf_weimwbm_we_o),
  .wbm_dat_o(buf_weimwbm_dat_o),
  .wbm_dat_i(buf_mwinwbs_dat_o),
  .wbm_adr_o(buf_weimwbm_adr_o),
  .wbm_ack_i(buf_mwinwbs_ack_o),
  .wbm_sel_o(buf_weimwbm_sel_o)
);

wb_memwindow mwincore3(
  .wb_clk_i(wb_clk),
  .wb_rst_i(wb_rst),

  .wb_cyc_i(buf_weimwbm_cyc_o),
  .wb_stb_i(buf_weimwbm_stb_o),
  .wb_we_i(buf_weimwbm_we_o),
  .wb_adr_i(buf_weimwbm_adr_o),
  .wb_sel_i(buf_weimwbm_sel_o),
  .wb_dat_i(buf_weimwbm_dat_o),
  .wb_dat_o(buf_mwinwbs_dat_o),
  .wb_ack_o(buf_mwinwbs_ack_o),

  .wbm_cyc_o(mwinwbm_cyc_o),
  .wbm_stb_o(mwinwbm_stb_o),
  .wbm_we_o(mwinwbm_we_o),
  .wbm_adr_o(mwinwbm_adr_o),
  .wbm_sel_o(mwinwbm_sel_o),
  .wbm_dat_o(mwinwbm_dat_o),
  .wbm_dat_i(mwinwbm_dat_i),
  .wbm_ack_i(mwinwbm_ack_i),
  .wbm_buswidth_i(mwinbuswidth)
);


/****************************************************************************
 * EXTERNAL BUS
 ****************************************************************************/

wire buswbs2_ack_o;
wire [15:0] buswbs2_dat_o;
reg buswbs2_en;

/* On this board, the first 4KB of 16 bit muxbus space are accessible in two
 * ways: directly at offset 0x11000, or through the memory window.  8 bit
 * cycles or cycles accessing above 0xfff must use the memwindow.
 *
 * The first 64KB of memwindow space are for using the external bus in 
 * 16 bit mode.  The next 64KB (adr_o[16] == 1) are for the external bus in
 * 8 bit mode.  For 8 bit mode, the memwindow core will emit 8 bit wishbone
 * cycles, but the external bus core expects 16 bit cycles with a wb_sel of
 * 2'b01 or 2'b10.  The adapter handles this adjustment.  This scheme allows
 * for memwindow optimization features on the 8 bit muxbus.  This will
 * almost double the bandwidth, in practice, to an off-board 8 bit SRAM.
 */

wire [15:0] buswbs2_dat16_o;
wire [31:0] buswbs2_adr_i;
wire [15:0] buswbs2_dat_i;
wire [1:0]  buswbs2_sel_i;

// Yes, the _i and _o in the adapter are kind of backwards, they are relative
// to the modules on the other end of the lines.
wb_8to16adapter adapter1(
  .wbm_dat_o(mwinwbm_dat_o[15:0]),
  .wbm_dat_i(buswbs2_dat_o),
  .wbm_adr_o(mwinwbm_adr_o),
  .wbm_sel_o(mwinwbm_sel_o[1:0]),

  .wbs_dat_i(buswbs2_dat_i),
  .wbs_dat_o(buswbs2_dat16_o),
  .wbs_adr_i(buswbs2_adr_i),
  .wbs_sel_i(buswbs2_sel_i),

  .toggle8bit_i(mwinwbm_adr_o[16])
);

reg buswbs_en;
wire [15:0] buswbs_dat_o;
wire buswbs_ack_o;
wire bus_en = bus_config[0];
wire [15:0] bus_dat;
wire bus_dat_oe;
wire bus_alen, bus_dir, bus_csn, bus_bhen;
isamuxbus_bridge externalbus(
  .wb_clk_i(wb_clk & externalbus_opt),
  .wb_rst_i(wb_rst),
  .wb1_cyc_i(weimwbm_cyc_o && buswbs_en),
  .wb1_stb_i(weimwbm_stb_o && buswbs_en),
  .wb1_we_i(weimwbm_we_o),
  .wb1_adr_i({4'h0,weimwbm_adr_o[11:0]}), // 12 bits of bus space
  .wb1_dat_i(weimwbm_dat_o),
  .wb1_sel_i(weimwbm_sel_o),
  .wb1_dat_o(buswbs_dat_o),
  .wb1_ack_o(buswbs_ack_o),

  .wb2_cyc_i(mwinwbm_cyc_o && buswbs2_en),
  .wb2_stb_i(mwinwbm_stb_o && buswbs2_en),
  .wb2_we_i(mwinwbm_we_o),
  .wb2_adr_i(buswbs2_adr_i[15:0]), // 16 bits of bus space
  .wb2_dat_i(buswbs2_dat_i),
  .wb2_sel_i(buswbs2_sel_i),
  .wb2_dat_o(buswbs2_dat16_o),
  .wb2_ack_o(buswbs2_ack_o),

  .bus_config_i(bus_config),
  .cs_8bit_i(mwinwbm_cyc_o ? mwinwbm_adr_o[16] : 0),
  .csn_o(bus_csn),
  .dir_o(bus_dir),
  .ale_o(ale),
  .bhen_o(bus_bhen),
  .waitn_i(bus_wait_padn),
  .mux_i(mux_ad_pad[15:0]),
  .mux_o(bus_dat),
  .mux_oe_o(bus_dat_oe)
);
assign bus_alen = !ale;


/****************************************************************************
 * IRQ controller 
 ****************************************************************************/
reg irqwbs_en;
wire [31:0] irqwbs_dat_o;
wire irqwbs_ack_o;
wire [3:0] irqwbs_sel_i = weimwbm_adr_o[1] ? 
  {weimwbm_sel_o, 2'b00} : {2'b00, weimwbm_sel_o};

wire [31:0] irqs;
assign irqs[0] = xuart_irq;
assign irqs[1] = !can0_irqn;
assign irqs[2] = !can1_irqn;
assign irqs[3] = dio_pad[0];
assign irqs[4] = dio_pad[1];
assign irqs[5] = dio_pad[2];
irq_controller irqcore(
  .wb_clk_i(wb_clk),
  .wb_rst_i(wb_rst),
  .wbs_cyc_i(weimwbm_cyc_o && irqwbs_en),
  .wbs_stb_i(weimwbm_stb_o && irqwbs_en),
  .wbs_we_i(weimwbm_we_o),
  .wbs_adr_i({weimwbm_adr_o[31:2], 2'b00}),
  .wbs_dat_i(weimwbm_dat_o),
  .wbs_sel_i(irqwbs_sel_i),
  .wbs_dat_o(irqwbs_dat_o),
  .wbs_ack_o(irqwbs_ack_o),
  
  .wbm_ack_i(1'b1),

  .irq_i(irqs),
  .irq_o(irq_controller_o)
);


/****************************************************************************
 * PADS ASSIGNMENTS
 ****************************************************************************/
wire can0_en = core_enable[0];
wire can1_en = core_enable[1];
wire xu0_en = core_enable[2];
wire xu3_en = core_enable[3];
wire xu4_en = core_enable[4];
wire xu5_en = core_enable[5];
wire tsc_en = core_enable[6];
wire spi_en = core_enable[7];
wire snd_en = core_enable[9];
wire irq5_en = core_enable[11];
wire irq6_en = core_enable[12];
wire irq7_en = core_enable[13];

assign dio_i[14:0]    = dio_pad[14:0];
assign dio_i[20:17]   = dio_pad[20:17];
assign dio_i[36:21]   = mux_ad_pad[15:0];
assign dio_i[41:37]   = {bus_wait_padn, bus_bhe_padn, bus_cs_padn, bus_dir_pad, bus_ale_padn};
assign dio_i[45:42]   = {spi_mosi_pad, spi_cs_padn, spi_clk_pad, spi_miso_pad};
assign dio_i[47:46]   = {can_txd_pad, can_rxd_pad};
assign dio_i[49:48]   = {uart0_txd_pad, uart0_rxd_pad};
assign dio_i[51:50]   = {uart3_txd_pad, uart3_rxd_pad};
assign dio_i[53:52]   = {uart4_txd_pad, uart4_rxd_pad};
assign dio_i[55:54]   = {uart5_txd_pad, uart5_rxd_pad};

reg [55:0] dio_reg, pad_o, pad_oe;

assign dio_pad[14:0] = dio_reg[14:0];
assign dio_pad[20:17] = dio_reg[20:17];
assign mux_ad_pad[15:0] = dio_reg[36:21];
assign bus_ale_padn = dio_reg[37];
assign bus_dir_pad = dio_reg[38];
assign bus_cs_padn = dio_reg[39];
assign bus_bhe_padn = dio_reg[40];
assign bus_wait_padn = dio_reg[41];
assign spi_miso_pad = dio_reg[42];
assign spi_clk_pad = dio_reg[43];
assign spi_cs_padn = dio_reg[44];
assign spi_mosi_pad = dio_reg[45];
assign {can_txd_pad, can_rxd_pad} = dio_reg[47:46];
assign {uart0_txd_pad, uart0_rxd_pad} = dio_reg[49:48];
assign {uart3_txd_pad, uart3_rxd_pad} = dio_reg[51:50];
assign {uart4_txd_pad, uart4_rxd_pad} = dio_reg[53:52];
assign {uart5_txd_pad, uart5_rxd_pad} = dio_reg[55:54];

integer i;

always @(*) begin
  for (i = 0; i <= 55; i = i + 1) begin
    dio_reg[i] = pad_oe[i] ? pad_o[i] : 1'bz;
  end
end

always @(*) begin
  pad_o = dio_o;
  pad_oe = dio_oe_o;
  if (counter_en[0]) pad_oe[4] = 1'b0;
  if (counter_en[1]) pad_oe[6] = 1'b0;
  if (bus_en) begin
    pad_o[36:21] = bus_dat;
    pad_o[40:37] = {bus_bhen, bus_csn, bus_dir, bus_alen};
    pad_oe[36:21] = {16{bus_dat_oe}};
    pad_oe[41:37] = 5'b01111;
  end
  if (can0_en) begin
    pad_o[47] = can0_tx;
    pad_oe[47:46] = 2'b10;
  end
  if (can1_en) begin
    pad_o[10] = can1_tx;
    pad_oe[11:10] = 2'b01;
  end
  if (xu3_en & xuart_active[1]) begin
    pad_o[51] = xuart_tx[1];
    pad_o[13] = xuart_txen[1];
    pad_oe[51:50] = 2'b10;
    pad_oe[13] = 1'b1;
  end
  if (xu4_en & xuart_active[2]) begin
    if (en_rs422) pad_oe[13] = 1'b0;
    else pad_oe[52] = 1'b0;
    pad_o[53] = xuart_tx[2];
    pad_o[14] = xuart_txen[2];
    pad_oe[53] = 1'b1;
    pad_oe[14] = 1'b1;
  end
  // uart 0 takes priority over 1 and 2 on TS-8510 DIO 13
  if (xu0_en & xuart_active[0]) begin
    if (en_ts8510_rs422) pad_oe[13] = 1'b0;
    else pad_oe[48] = 1'b0;
    pad_o[49] = xuart_tx[0];
    pad_o[12] = xuart_txen[0];
    pad_oe[49] = 1'b1;
    pad_oe[12] = 1'b1;
  end
  if (xu5_en & xuart_active[3]) begin
    pad_o[55] = xuart_tx[3];
    pad_o[7] = xuart_txen[3];
    pad_oe[55:54] = 2'b10;
    pad_oe[7] = 1'b1;
  end
  if (spi_en) begin
    pad_o[45] = so;
    pad_oe[45] = 1'b1;
    pad_o[44] = 1'b0;
    pad_oe[44] = !csn[0] || !csn_q[0];
    if (hispeed) pad_o[43] = pll_72mhz_shifted;
    else pad_o[43] = sck;
    pad_oe[43:42] = 2'b10;
  end
  if (adc_active) begin
    pad_o[7:6] = 2'b00;
    pad_oe[7] = !adc_scl;
    pad_oe[6] = !adc_sda;
    if (an_sel_sel == 2'b01) begin
      pad_o[8] = an_sel;
      pad_oe[8] = 1'b1;
    end
    else if (an_sel_sel == 2'b10) begin
      pad_o[31] = an_sel;
      pad_oe[31] = 1'b1;
    end
  end
  if (tsc_en) begin
    pad_o[36:35] = {touch_clk, touch_mosi};
    pad_oe[36:34] = 3'b110;
    pad_o[33] = 1'b0;
    pad_oe[33] = !ad7843_csn;
    pad_oe[32] = 1'b0;
  end
  if (irq5_en) pad_oe[0] = 1'b0;
  if (irq6_en) pad_oe[1] = 1'b0;
  if (irq7_en) pad_oe[2] = 1'b0;
  if (reset) pad_oe[38] = 1'b0;
  if (bbclk) begin
    pad_o[3] = clk_12mhz;
    pad_oe[3] = 1'b1;
  end
  if (snd_en) begin
    pad_o[20] = fpga_24mhz_pad;
    pad_oe[20] = 1'b1;
  end
end

always @(*) begin
  if (en_ts8510_rs422) xuart_rx[0] = dio_pad[13];
  else xuart_rx[0] = uart0_rxd_pad;
  xuart_rx[1] = uart3_rxd_pad;
  if (en_rs422) xuart_rx[2] = dio_pad[13]; 
  else xuart_rx[2] = uart4_rxd_pad;
  xuart_rx[3] = uart5_rxd_pad;
end


/****************************************************************************
 * Memory window address decode
 ****************************************************************************/

/* Address decode:
 * 0 to 0xFFFF            16 bit external bus space
 * 0x10000 to 0x1FFFF     8 bit external bus space
 * higher = reserved
 * Memory window currently decodes 27 bits of address.
 *
 * mwinbuswidth settings:
 *     0 - use internal setting
 *     1 - 32 bit
 *     2 - 16 bit
 *     3 - 8 bit
 */

reg [3:0] decodes;
always @(posedge wb_clk or posedge wb_rst) begin
  if (wb_rst) decodes <= 4'b0000;
  else if (mwinwbm_adr_o < 32'h00020000) decodes <= 4'b0001;
  // Other cores with large address spaces could be added here.
  else decodes <= 4'b0000;
end

always @(*) begin
  buswbs2_en = 1'b0;
  mwinwbm_dat_i = 32'hxxxxxxxx;
  mwinwbm_ack_i = 1'b1;
  mwinbuswidth = 2'b00;

  if (decodes[0]) begin
    buswbs2_en = 1'b1;
    mwinwbm_dat_i[15:0] = buswbs2_dat_o;
    mwinwbm_ack_i = buswbs2_ack_o;
    mwinbuswidth = mwinwbm_adr_o[16] ? 2'b11 : 2'b10;
  end

end


/****************************************************************************
 * TOP-LEVEL ADDRESS DECODE FOR WEIM-WISHBONE BRIDGE
 ****************************************************************************/
always @(*) begin
  ramwbs1_en =		ts4800_decodes[0]; 	//0xb0000000
  scwbs_en =		ts4800_decodes[1]; 	//0xb0010000 
  buswbs_en =		ts4800_decodes[2]; 	//0xb0011000
  tswbs_en =		ts4800_decodes[3]; 	//0xb0012000
  spiwbs_en =		ts4800_decodes[4]; 	//0xb0013000
  motwbs_en =		ts4800_decodes[5];	//0xb0014000
  irqwbs_en =		ts4800_decodes[6];	//0xb0015000
  adcwbs_en =		ts4800_decodes[7];	//0xb0016000
  mwinwbs_en =		ts4800_decodes[8];	//0xb0017000
  xuwbs_en =		ts4800_decodes[9];	//0xb0018000
  mwcan0wbs_en =	ts4800_decodes[11];	//0xb001a000
  mwcan1wbs_en  =	ts4800_decodes[12];	//0xb001b000

  weimwbm_dat_i = 16'hxxxx;
  weimwbm_ack_i = 1'b1;

  if (ts4800_decodes[0]) begin
    weimwbm_dat_i = ramwbs1_dat16_o;
    weimwbm_ack_i = ramwbs1_ack_o;
  end else if (ts4800_decodes[1]) begin
    weimwbm_dat_i = scwbs_dat_o;
    weimwbm_ack_i = scwbs_ack_o;
  end else if (ts4800_decodes[2]) begin
    weimwbm_dat_i = buswbs_dat_o;
    weimwbm_ack_i = buswbs_ack_o | !externalbus_opt;
  end else if (ts4800_decodes[3]) begin
    weimwbm_dat_i = tswbs_dat_o;
    weimwbm_ack_i = tswbs_ack_o | !touchscreen_opt;
  end else if (ts4800_decodes[4]) begin
    weimwbm_dat_i = spiwbs_dat_o;
    weimwbm_ack_i = spiwbs_ack_o;
  end else if (ts4800_decodes[5]) begin
    weimwbm_dat_i = motwbs_dat_o;
    weimwbm_ack_i = motwbs_ack_o;
  end else if (ts4800_decodes[6]) begin
    weimwbm_dat_i = irqwbs_dat_o;
    weimwbm_ack_i = irqwbs_ack_o;
  end else if (ts4800_decodes[7]) begin
    weimwbm_dat_i = adcwbs_dat_o;
    weimwbm_ack_i = adcwbs_ack_o;
  end else if (ts4800_decodes[8]) begin
    weimwbm_dat_i = mwinwbs_dat_o;
    weimwbm_ack_i = mwinwbs_ack_o;
  end else if (ts4800_decodes[9]) begin
    weimwbm_dat_i = xuartwbs_dat_o;
    weimwbm_ack_i = xuartwbs_ack_o;
  end else if (ts4800_decodes[11]) begin
    weimwbm_dat_i = mwcan0wbs_dat_o;
    weimwbm_ack_i = mwcan0wbs_ack_o;
  end else if (ts4800_decodes[12]) begin
    weimwbm_dat_i = mwcan1wbs_dat_o;
    weimwbm_ack_i = mwcan1wbs_ack_o;
  end

end


endmodule


/* 8to16adapter does one of two things:
 * If toggle8bit_i is 0, it passes the 16 bit wishbone bus straight through.
 * Otherwise, it converts wbm 8 bit cycles into wbs 16 bit cycles.
 */

module wb_8to16adapter (
  wbm_dat_o,
  wbm_dat_i,
  wbm_adr_o,
  wbm_sel_o,

  wbs_dat_i,
  wbs_dat_o,
  wbs_adr_i,
  wbs_sel_i,

  toggle8bit_i
);

input [15:0] wbm_dat_o;
output [15:0] wbm_dat_i;
input [31:0] wbm_adr_o;
input [1:0] wbm_sel_o;

output [15:0] wbs_dat_i;
input [15:0] wbs_dat_o;
output [31:0] wbs_adr_i;
output [1:0] wbs_sel_i;

input toggle8bit_i;

wire t = toggle8bit_i;
wire e = !wbm_adr_o[0];

assign wbm_dat_i = (t && !e) ? {8'hxx, wbs_dat_o[15:8]} : wbs_dat_o;
assign wbs_dat_i = t ? {wbm_dat_o[7:0], wbm_dat_o[7:0]} : wbm_dat_o;
assign wbs_adr_i[31:1] = wbm_adr_o[31:1];
assign wbs_adr_i[0] = t ? 1'b0 : wbm_adr_o[0];
assign wbs_sel_i = t ? e ? 2'b01 : 2'b10 : wbm_sel_o;

endmodule
